/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Mateusz Bilski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_SOCKET_H
#define SMART_TOOLS_SOCKET_H

#include <sys/types.h>
#include <sys/socket.h>

#include "stbuffer.h"

/*!
	\brief This class allows for doing basic operations on sockets.
	
	It is used by StClient and StServer.
*/

class StSocket
{
	public:
		/*! 
			Create an endpoint for communication.
			\param domain specifies a communication domain.
		*/
		StSocket( int domain = PF_UNIX ){ init(); lock(); create( domain ); };
		StSocket( const StSocket& copy );
		~StSocket() { unlock(); clean(); };
		
		/*! Close the socket. */		
		void close(){ ::close( *socket ); }
		
		/*! 
			Set the OS socket on given ST socket.
			\note You should close the socket before performing this function.
		*/
		void setSocket( int socket ) { close(); *(this->socket) = socket; }
		
		/*! \return the socket. */
		int getSocket() const { return *socket; }
		
		/*! 
			Create an endpoint for communication.
			\param domain specifies a communication domain.
		*/
		void create( int domain );
		
		/*! First close the socket, and than create a new one 
		    with the domain which was given to the constructor. */
		void recreate(){ close(); create( *domain ); }
		
		/*! \return the domain which was given to the constructor. */
		int getDomain() { return *domain; }
		
		StSocket& operator=( const StSocket& copy );
	protected:
		int* socket;
		int* domain;
		
		unsigned* instances;
		
		void init();
		void clean();
		
		void lock() { /*_printf("shared (%p): instance++: ", this);*/ (*instances)++; /*_printf("%d\n", *instances);*/ }
		void unlock() { assert( *instances > 0 ); /*_printf("shared (%p): instance--: ", this);*/ (*instances)--; /*_printf("%d\n", *instances);*/ }
};

#endif //SMART_TOOLS_SOCKET_H

