/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_STRING_LIST_H
#define SMART_TOOLS_STRING_LIST_H

#include "stfile.h"
#include "ststring.h"
#include "stvaluelist.h"

class StStringList : public StValueList<StString>
{
	
	public:
		StStringList();
		StStringList( const StString& str ){ init(); push_back(str); }
		StStringList( const StValueList<StString>& list ){ memory = list.memory; itemCount = list.itemCount; }
		
		/** Load file lines into string list. It doesn't erase existing strings but appends new ones. */
		bool load( StFile file );
		/** Save all strings into a file as seperate lines. If file exists, its context will be erased. */
		bool save( StFile file );
		
		static StStringList split( const StString& sep, const StString& str, bool allowEmptyEntires=false );
		/** Join all strings into one seperated by glue and return the result in one StString. */
		StString join( const StString& glue ) const;
		
		/** Removes empty lines. */
		StStringList& removeEmpty( bool ignoreSpaces=true, bool ignoreTabs=true );

		/**
			Sorts all strings in list alphabetically or reverse to alphabet.
			\param alphabetically  - do sort alphabetically or reversed?
		*/
		void sort( bool alphabetically=true );
		
	private:
		void init();
};

/*! \class StStringList
	\brief StValueList<StString> with some new methods.
	
	This class is StValueList<StString> with possibility of load() lines of file, and
	save() contents to another file. You can also split() existing StString by seperator or
	join() all strings into one.
*/

/*! \fn static StStringList StStringList::split( const StString& sep, const StString& str, bool allowEmptyEntires=false )
	Split string to StStringList by sep. If allowEmptyEntires is true,
	then empty fields between two seperators will also be included.
	
	<b><i>Example</i></b>:
	\include ststringlist-split.cpp
*/

#endif //SMART_TOOLS_STRING_LIST_H
